/* $Id$ -*- C++ -*-
 */

#include "contour.h"
#include <cstring>

void find_area(Station* s)
{
  double area_p60 = 0;
  double area_i40 = 0;
  double area_i54 = 0;
  double area_i100 = 0;
  double sumsq = 0;
  int count = 0;

  Station::iterator i = s->end()-1;
  double r1_p60 = i->_p60;
  double r1_i40 = i->_i40;
  double r1_i54 = i->_i54;
  double r1_i100 = i->_i100;
  Angle b1 = i->_angle;

  for (i = s->begin(); i != s->end(); ++i) {    
    Angle b2 = i->_angle;
    double a = ((b2-b1).degrees())/360;
    b1 = b2;

    double r2_p60 = i->_p60;
    double r = (r2_p60 + r1_p60) / 2;
    area_p60 += a * M_PI * r * r;
    r1_p60 = r2_p60;
    
    double r2_i40 = i->_i40;
    r = (r2_i40 + r1_i40) / 2;
    area_i40 += M_PI * a * r * r;
    r1_i40 = r2_i40;

    double r2_i54 = i->_i54;
    r = (r2_i54 + r1_i54) / 2;
    area_i54 += M_PI * a * r * r;
    r1_i54 = r2_i54;

    double r2_i100 = i->_i100;
    r = (r2_i100 + r1_i100) / 2;
    area_i100 += M_PI * a * r * r;
    r1_i100 = r2_i100;

    double field = i->_antenna_rel_field;
    sumsq += field * field;
    ++count;
  }

  double rms = sqrt(sumsq/count);

  std::cout.setf(std::ios_base::fixed, std::ios_base::floatfield);
  std::cout.precision(0);
  std::cout
    << " p60=" << area_p60
    << " i40=" << area_i40
    << " i54=" << area_i54
    << " i100=" << area_i100;
  std::cout.precision(4);
  std::cout << " rms=" << rms;
}

void max_slope(Station* s)
{
  Station::iterator i = s->end()-1;
  double db1 = i->_antenna_rel_db;
  Angle b1 = i->_angle;
  double hi_slope = 0;
  double angle_1 = 0;
  double angle_2 = 0;
  
  for (i = s->begin(); i != s->end(); ++i) {    
    Angle b2 = i->_angle;
    double db2 = i->_antenna_rel_db;
    double slope = std::abs((db2 - db1) / (b2 - b1).degrees()) * 10;
    if (slope > hi_slope) {
      hi_slope = slope;
      angle_1 = b1.degrees();
      angle_2 = b2.degrees();
    }else{
    }
    b1 = b2;
    db1 = db2;
  }
  std::cout.setf(std::ios_base::fixed);
  std::cout.precision(2);
  std::cout << hi_slope << "\t";
  std::cout.precision(0);
  std::cout << angle_1 << '-' << angle_2;
}

int main(int argc, char* argv[])
{
  if (argc <= 2) {
    std::cerr << "no args?\n";
    exit(1);
  }else{
  }
  if (strcmp(argv[1], "-area") == 0) {
  }else if (strcmp(argv[1], "-slope") == 0) {
  }else{
    std::cout << "error: " << argv[1] << " unrecognized\n";
  }

  for (int ii=2; ii<argc; ++ii) {
    std::ifstream f2(argv[ii]);
    if (!f2) {
      std::cerr << "can't open file: " << argv[ii] << ", skipping\n";
      continue;
    }
    Station s(f2);
    if (s.begin() == s.end()) {
      std::cout << "no data";
    }else if (strcmp(argv[1], "-area") == 0) {
      find_area(&s);
    }else if (strcmp(argv[1], "-slope") == 0) {
      max_slope(&s);
    }else{
    }
    std::cout << '\t' << argv[ii] << '\n';
  }
  
  return 0;
}
